# Autogenerated by nbdev

d = { 'settings': { 'branch': 'master',
                'doc_baseurl': '/nbstata',
                'doc_host': 'https://hugetim.github.io',
                'git_url': 'https://github.com/hugetim/nbstata',
                'lib_path': 'nbstata'},
  'syms': { 'nbstata.browse': { 'nbstata.browse._get_pos_stata_obs_num': ('browse.html#_get_pos_stata_obs_num', 'nbstata/browse.py'),
                                'nbstata.browse._get_start_end_strs': ('browse.html#_get_start_end_strs', 'nbstata/browse.py'),
                                'nbstata.browse._parse_browse_magic_syntax': ( 'browse.html#_parse_browse_magic_syntax',
                                                                               'nbstata/browse.py'),
                                'nbstata.browse._parse_df_params': ('browse.html#_parse_df_params', 'nbstata/browse.py'),
                                'nbstata.browse.browse_df_params': ('browse.html#browse_df_params', 'nbstata/browse.py'),
                                'nbstata.browse.display_df_as_ipydatagrid': ('browse.html#display_df_as_ipydatagrid', 'nbstata/browse.py'),
                                'nbstata.browse.get_df': ('browse.html#get_df', 'nbstata/browse.py'),
                                'nbstata.browse.headtail_df_params': ('browse.html#headtail_df_params', 'nbstata/browse.py'),
                                'nbstata.browse.headtail_get_df': ('browse.html#headtail_get_df', 'nbstata/browse.py'),
                                'nbstata.browse.in_range': ('browse.html#in_range', 'nbstata/browse.py'),
                                'nbstata.browse.parse_browse_magic': ('browse.html#parse_browse_magic', 'nbstata/browse.py'),
                                'nbstata.browse.set_ipydatagrid_height': ('browse.html#set_ipydatagrid_height', 'nbstata/browse.py')},
            'nbstata.cell': { 'nbstata.cell.Cell': ('cell.html#cell', 'nbstata/cell.py'),
                              'nbstata.cell.Cell.__init__': ('cell.html#cell.__init__', 'nbstata/cell.py'),
                              'nbstata.cell.Cell.run': ('cell.html#cell.run', 'nbstata/cell.py')},
            'nbstata.code_utils': { 'nbstata.code_utils._block': ('code_utils.html#_block', 'nbstata/code_utils.py'),
                                    'nbstata.code_utils._end_block_followed_by_non_comment_block': ( 'code_utils.html#_end_block_followed_by_non_comment_block',
                                                                                                     'nbstata/code_utils.py'),
                                    'nbstata.code_utils._is_not_cr_delimiter': ( 'code_utils.html#_is_not_cr_delimiter',
                                                                                 'nbstata/code_utils.py'),
                                    'nbstata.code_utils._lex_tokens': ('code_utils.html#_lex_tokens', 'nbstata/code_utils.py'),
                                    'nbstata.code_utils._prog_blocks': ('code_utils.html#_prog_blocks', 'nbstata/code_utils.py'),
                                    'nbstata.code_utils._remove_prefixes': ('code_utils.html#_remove_prefixes', 'nbstata/code_utils.py'),
                                    'nbstata.code_utils._replace_delimiter': ( 'code_utils.html#_replace_delimiter',
                                                                               'nbstata/code_utils.py'),
                                    'nbstata.code_utils._replace_tabs': ('code_utils.html#_replace_tabs', 'nbstata/code_utils.py'),
                                    'nbstata.code_utils._startswith_stata_abbrev': ( 'code_utils.html#_startswith_stata_abbrev',
                                                                                     'nbstata/code_utils.py'),
                                    'nbstata.code_utils.break_out_prog_blocks': ( 'code_utils.html#break_out_prog_blocks',
                                                                                  'nbstata/code_utils.py'),
                                    'nbstata.code_utils.ending_code_version': ( 'code_utils.html#ending_code_version',
                                                                                'nbstata/code_utils.py'),
                                    'nbstata.code_utils.ending_sc_delimiter': ( 'code_utils.html#ending_sc_delimiter',
                                                                                'nbstata/code_utils.py'),
                                    'nbstata.code_utils.ends_in_comment_block': ( 'code_utils.html#ends_in_comment_block',
                                                                                  'nbstata/code_utils.py'),
                                    'nbstata.code_utils.is_start_of_program_block': ( 'code_utils.html#is_start_of_program_block',
                                                                                      'nbstata/code_utils.py'),
                                    'nbstata.code_utils.remove_comments': ('code_utils.html#remove_comments', 'nbstata/code_utils.py'),
                                    'nbstata.code_utils.standardize_code': ('code_utils.html#standardize_code', 'nbstata/code_utils.py'),
                                    'nbstata.code_utils.valid_single_line_code': ( 'code_utils.html#valid_single_line_code',
                                                                                   'nbstata/code_utils.py')},
            'nbstata.completion_env': { 'nbstata.completion_env.CompletionEnv': ( 'completion_env.html#completionenv',
                                                                                  'nbstata/completion_env.py'),
                                        'nbstata.completion_env.CompletionEnv.__init__': ( 'completion_env.html#completionenv.__init__',
                                                                                           'nbstata/completion_env.py'),
                                        'nbstata.completion_env.CompletionEnv._last_line_first_word': ( 'completion_env.html#completionenv._last_line_first_word',
                                                                                                        'nbstata/completion_env.py'),
                                        'nbstata.completion_env.CompletionEnv._scalar_f_pos_rcomp': ( 'completion_env.html#completionenv._scalar_f_pos_rcomp',
                                                                                                      'nbstata/completion_env.py'),
                                        'nbstata.completion_env.CompletionEnv._start_of_last_chunk': ( 'completion_env.html#completionenv._start_of_last_chunk',
                                                                                                       'nbstata/completion_env.py'),
                                        'nbstata.completion_env.CompletionEnv._start_of_last_word': ( 'completion_env.html#completionenv._start_of_last_word',
                                                                                                      'nbstata/completion_env.py'),
                                        'nbstata.completion_env.CompletionEnv.get_env': ( 'completion_env.html#completionenv.get_env',
                                                                                          'nbstata/completion_env.py'),
                                        'nbstata.completion_env.Env': ('completion_env.html#env', 'nbstata/completion_env.py'),
                                        'nbstata.completion_env._ends_in_a_comment': ( 'completion_env.html#_ends_in_a_comment',
                                                                                       'nbstata/completion_env.py'),
                                        'nbstata.completion_env._ends_in_string_literal': ( 'completion_env.html#_ends_in_string_literal',
                                                                                            'nbstata/completion_env.py'),
                                        'nbstata.completion_env._last_token': ( 'completion_env.html#_last_token',
                                                                                'nbstata/completion_env.py'),
                                        'nbstata.completion_env._last_token_full_string': ( 'completion_env.html#_last_token_full_string',
                                                                                            'nbstata/completion_env.py'),
                                        'nbstata.completion_env._lex_tokens': ( 'completion_env.html#_lex_tokens',
                                                                                'nbstata/completion_env.py')},
            'nbstata.completions': { 'nbstata.completions.CompletionsManager': ( 'completions.html#completionsmanager',
                                                                                 'nbstata/completions.py'),
                                     'nbstata.completions.CompletionsManager.__init__': ( 'completions.html#completionsmanager.__init__',
                                                                                          'nbstata/completions.py'),
                                     'nbstata.completions.CompletionsManager.do': ( 'completions.html#completionsmanager.do',
                                                                                    'nbstata/completions.py'),
                                     'nbstata.completions.CompletionsManager.get': ( 'completions.html#completionsmanager.get',
                                                                                     'nbstata/completions.py'),
                                     'nbstata.completions.CompletionsManager.get_file_paths': ( 'completions.html#completionsmanager.get_file_paths',
                                                                                                'nbstata/completions.py'),
                                     'nbstata.completions.CompletionsManager.get_globals': ( 'completions.html#completionsmanager.get_globals',
                                                                                             'nbstata/completions.py')},
            'nbstata.config': { 'nbstata.config.Config': ('config.html#config', 'nbstata/config.py'),
                                'nbstata.config.Config.__init__': ('config.html#config.__init__', 'nbstata/config.py'),
                                'nbstata.config.Config._display_and_clear_update_errors': ( 'config.html#config._display_and_clear_update_errors',
                                                                                            'nbstata/config.py'),
                                'nbstata.config.Config._get_config_env': ('config.html#config._get_config_env', 'nbstata/config.py'),
                                'nbstata.config.Config._update_backup_graph_size': ( 'config.html#config._update_backup_graph_size',
                                                                                     'nbstata/config.py'),
                                'nbstata.config.Config.display_status': ('config.html#config.display_status', 'nbstata/config.py'),
                                'nbstata.config.Config.echo': ('config.html#config.echo', 'nbstata/config.py'),
                                'nbstata.config.Config.init_stata': ('config.html#config.init_stata', 'nbstata/config.py'),
                                'nbstata.config.Config.noecho': ('config.html#config.noecho', 'nbstata/config.py'),
                                'nbstata.config.Config.process_config_file': ( 'config.html#config.process_config_file',
                                                                               'nbstata/config.py'),
                                'nbstata.config.Config.set_graph_size': ('config.html#config.set_graph_size', 'nbstata/config.py'),
                                'nbstata.config.Config.splash': ('config.html#config.splash', 'nbstata/config.py'),
                                'nbstata.config.Config.update': ('config.html#config.update', 'nbstata/config.py'),
                                'nbstata.config.Config.update_graph_config': ( 'config.html#config.update_graph_config',
                                                                               'nbstata/config.py'),
                                'nbstata.config._edition': ('config.html#_edition', 'nbstata/config.py'),
                                'nbstata.config._find_path': ('config.html#_find_path', 'nbstata/config.py'),
                                'nbstata.config._get_config_settings': ('config.html#_get_config_settings', 'nbstata/config.py'),
                                'nbstata.config._mac_find_path': ('config.html#_mac_find_path', 'nbstata/config.py'),
                                'nbstata.config._other_find_path': ('config.html#_other_find_path', 'nbstata/config.py'),
                                'nbstata.config._set_graph_size': ('config.html#_set_graph_size', 'nbstata/config.py'),
                                'nbstata.config._win_find_path': ('config.html#_win_find_path', 'nbstata/config.py'),
                                'nbstata.config.find_dir_edition': ('config.html#find_dir_edition', 'nbstata/config.py'),
                                'nbstata.config.find_edition': ('config.html#find_edition', 'nbstata/config.py'),
                                'nbstata.config.launch_stata': ('config.html#launch_stata', 'nbstata/config.py'),
                                'nbstata.config.old_user_config_path': ('config.html#old_user_config_path', 'nbstata/config.py'),
                                'nbstata.config.set_graph_format': ('config.html#set_graph_format', 'nbstata/config.py'),
                                'nbstata.config.set_pystata_path': ('config.html#set_pystata_path', 'nbstata/config.py'),
                                'nbstata.config.xdg_user_config_path': ('config.html#xdg_user_config_path', 'nbstata/config.py')},
            'nbstata.inspect': {'nbstata.inspect.get_inspect': ('inspect.html#get_inspect', 'nbstata/inspect.py')},
            'nbstata.install': { 'nbstata.install._conf_default': ('install.html#_conf_default', 'nbstata/install.py'),
                                 'nbstata.install._conf_path': ('install.html#_conf_path', 'nbstata/install.py'),
                                 'nbstata.install._find_stata': ('install.html#_find_stata', 'nbstata/install.py'),
                                 'nbstata.install._is_root': ('install.html#_is_root', 'nbstata/install.py'),
                                 'nbstata.install.create_conf_if_needed': ('install.html#create_conf_if_needed', 'nbstata/install.py'),
                                 'nbstata.install.install_kernel_spec': ('install.html#install_kernel_spec', 'nbstata/install.py'),
                                 'nbstata.install.main': ('install.html#main', 'nbstata/install.py')},
            'nbstata.kernel': { 'nbstata.kernel.PyStataKernel': ('kernel.html#pystatakernel', 'nbstata/kernel.py'),
                                'nbstata.kernel.PyStataKernel.__init__': ('kernel.html#pystatakernel.__init__', 'nbstata/kernel.py'),
                                'nbstata.kernel.PyStataKernel.do_complete': ('kernel.html#pystatakernel.do_complete', 'nbstata/kernel.py'),
                                'nbstata.kernel.PyStataKernel.do_execute': ('kernel.html#pystatakernel.do_execute', 'nbstata/kernel.py'),
                                'nbstata.kernel.PyStataKernel.do_history': ('kernel.html#pystatakernel.do_history', 'nbstata/kernel.py'),
                                'nbstata.kernel.PyStataKernel.do_inspect': ('kernel.html#pystatakernel.do_inspect', 'nbstata/kernel.py'),
                                'nbstata.kernel.PyStataKernel.do_is_complete': ( 'kernel.html#pystatakernel.do_is_complete',
                                                                                 'nbstata/kernel.py'),
                                'nbstata.kernel.PyStataKernel.init_session': ( 'kernel.html#pystatakernel.init_session',
                                                                               'nbstata/kernel.py'),
                                'nbstata.kernel.PyStataKernel.post_do_hook': ( 'kernel.html#pystatakernel.post_do_hook',
                                                                               'nbstata/kernel.py'),
                                'nbstata.kernel._format_user_obj': ('kernel.html#_format_user_obj', 'nbstata/kernel.py'),
                                'nbstata.kernel._handle_stata_error': ('kernel.html#_handle_stata_error', 'nbstata/kernel.py'),
                                'nbstata.kernel._handle_stata_import_error': ( 'kernel.html#_handle_stata_import_error',
                                                                               'nbstata/kernel.py'),
                                'nbstata.kernel._handle_stata_init_error': ('kernel.html#_handle_stata_init_error', 'nbstata/kernel.py'),
                                'nbstata.kernel._stata_error_reply': ('kernel.html#_stata_error_reply', 'nbstata/kernel.py'),
                                'nbstata.kernel._user_expressions': ('kernel.html#_user_expressions', 'nbstata/kernel.py'),
                                'nbstata.kernel.print_stata_error': ('kernel.html#print_stata_error', 'nbstata/kernel.py')},
            'nbstata.magics': { 'nbstata.magics.Frame': ('magics.html#frame', 'nbstata/magics.py'),
                                'nbstata.magics.Frame.__enter__': ('magics.html#frame.__enter__', 'nbstata/magics.py'),
                                'nbstata.magics.Frame.__exit__': ('magics.html#frame.__exit__', 'nbstata/magics.py'),
                                'nbstata.magics.Frame.__init__': ('magics.html#frame.__init__', 'nbstata/magics.py'),
                                'nbstata.magics.StataMagics': ('magics.html#statamagics', 'nbstata/magics.py'),
                                'nbstata.magics.StataMagics._do_magic': ('magics.html#statamagics._do_magic', 'nbstata/magics.py'),
                                'nbstata.magics.StataMagics._get_help_html': ( 'magics.html#statamagics._get_help_html',
                                                                               'nbstata/magics.py'),
                                'nbstata.magics.StataMagics._headtail_html': ( 'magics.html#statamagics._headtail_html',
                                                                               'nbstata/magics.py'),
                                'nbstata.magics.StataMagics._magic_frheadtail': ( 'magics.html#statamagics._magic_frheadtail',
                                                                                  'nbstata/magics.py'),
                                'nbstata.magics.StataMagics._magic_headtail': ( 'magics.html#statamagics._magic_headtail',
                                                                                'nbstata/magics.py'),
                                'nbstata.magics.StataMagics._parse_code_for_magic': ( 'magics.html#statamagics._parse_code_for_magic',
                                                                                      'nbstata/magics.py'),
                                'nbstata.magics.StataMagics._unabbrev_magic_name': ( 'magics.html#statamagics._unabbrev_magic_name',
                                                                                     'nbstata/magics.py'),
                                'nbstata.magics.StataMagics.magic': ('magics.html#statamagics.magic', 'nbstata/magics.py'),
                                'nbstata.magics.StataMagics.magic_browse': ('magics.html#statamagics.magic_browse', 'nbstata/magics.py'),
                                'nbstata.magics.StataMagics.magic_delimit': ('magics.html#statamagics.magic_delimit', 'nbstata/magics.py'),
                                'nbstata.magics.StataMagics.magic_echo': ('magics.html#statamagics.magic_echo', 'nbstata/magics.py'),
                                'nbstata.magics.StataMagics.magic_frbrowse': ( 'magics.html#statamagics.magic_frbrowse',
                                                                               'nbstata/magics.py'),
                                'nbstata.magics.StataMagics.magic_frhead': ('magics.html#statamagics.magic_frhead', 'nbstata/magics.py'),
                                'nbstata.magics.StataMagics.magic_frtail': ('magics.html#statamagics.magic_frtail', 'nbstata/magics.py'),
                                'nbstata.magics.StataMagics.magic_head': ('magics.html#statamagics.magic_head', 'nbstata/magics.py'),
                                'nbstata.magics.StataMagics.magic_help': ('magics.html#statamagics.magic_help', 'nbstata/magics.py'),
                                'nbstata.magics.StataMagics.magic_locals': ('magics.html#statamagics.magic_locals', 'nbstata/magics.py'),
                                'nbstata.magics.StataMagics.magic_noecho': ('magics.html#statamagics.magic_noecho', 'nbstata/magics.py'),
                                'nbstata.magics.StataMagics.magic_quietly': ('magics.html#statamagics.magic_quietly', 'nbstata/magics.py'),
                                'nbstata.magics.StataMagics.magic_set': ('magics.html#statamagics.magic_set', 'nbstata/magics.py'),
                                'nbstata.magics.StataMagics.magic_status': ('magics.html#statamagics.magic_status', 'nbstata/magics.py'),
                                'nbstata.magics.StataMagics.magic_tail': ('magics.html#statamagics.magic_tail', 'nbstata/magics.py'),
                                'nbstata.magics._clean_error_message': ('magics.html#_clean_error_message', 'nbstata/magics.py'),
                                'nbstata.magics._construct_abbrev_dict': ('magics.html#_construct_abbrev_dict', 'nbstata/magics.py'),
                                'nbstata.magics._formatted_local_list': ('magics.html#_formatted_local_list', 'nbstata/magics.py'),
                                'nbstata.magics._get_html_data': ('magics.html#_get_html_data', 'nbstata/magics.py'),
                                'nbstata.magics._get_new_settings': ('magics.html#_get_new_settings', 'nbstata/magics.py'),
                                'nbstata.magics._parse_frame_prefix': ('magics.html#_parse_frame_prefix', 'nbstata/magics.py'),
                                'nbstata.magics._parse_magic_name_code': ('magics.html#_parse_magic_name_code', 'nbstata/magics.py'),
                                'nbstata.magics._process_new_settings': ('magics.html#_process_new_settings', 'nbstata/magics.py'),
                                'nbstata.magics.print_kernel': ('magics.html#print_kernel', 'nbstata/magics.py')},
            'nbstata.misc_utils': { 'nbstata.misc_utils.Timer': ('misc_utils.html#timer', 'nbstata/misc_utils.py'),
                                    'nbstata.misc_utils.Timer.__enter__': ('misc_utils.html#timer.__enter__', 'nbstata/misc_utils.py'),
                                    'nbstata.misc_utils.Timer.__exit__': ('misc_utils.html#timer.__exit__', 'nbstata/misc_utils.py'),
                                    'nbstata.misc_utils.Timer.start': ('misc_utils.html#timer.start', 'nbstata/misc_utils.py'),
                                    'nbstata.misc_utils.Timer.stop': ('misc_utils.html#timer.stop', 'nbstata/misc_utils.py'),
                                    'nbstata.misc_utils.print_red': ('misc_utils.html#print_red', 'nbstata/misc_utils.py')},
            'nbstata.noecho': { 'nbstata.noecho._after_local_dict': ('noecho.html#_after_local_dict', 'nbstata/noecho.py'),
                                'nbstata.noecho._local_names_from_sreturn': ('noecho.html#_local_names_from_sreturn', 'nbstata/noecho.py'),
                                'nbstata.noecho._restore_locals_and_clear_sreturn': ( 'noecho.html#_restore_locals_and_clear_sreturn',
                                                                                      'nbstata/noecho.py'),
                                'nbstata.noecho._run_as_program_w_locals_sreturned': ( 'noecho.html#_run_as_program_w_locals_sreturned',
                                                                                       'nbstata/noecho.py'),
                                'nbstata.noecho.run_as_program_w_locals': ('noecho.html#run_as_program_w_locals', 'nbstata/noecho.py'),
                                'nbstata.noecho.run_noecho': ('noecho.html#run_noecho', 'nbstata/noecho.py'),
                                'nbstata.noecho.run_non_prog_noecho': ('noecho.html#run_non_prog_noecho', 'nbstata/noecho.py'),
                                'nbstata.noecho.run_prog_noecho': ('noecho.html#run_prog_noecho', 'nbstata/noecho.py')},
            'nbstata.pandas': { 'nbstata.pandas._better_dataframe': ('pandas.html#_better_dataframe', 'nbstata/pandas.py'),
                                'nbstata.pandas._simple_dataframe_from_stata': ( 'pandas.html#_simple_dataframe_from_stata',
                                                                                 'nbstata/pandas.py'),
                                'nbstata.pandas.better_dataframe_from_stata': ( 'pandas.html#better_dataframe_from_stata',
                                                                                'nbstata/pandas.py'),
                                'nbstata.pandas.better_pdataframe_from_data': ( 'pandas.html#better_pdataframe_from_data',
                                                                                'nbstata/pandas.py'),
                                'nbstata.pandas.better_pdataframe_from_frame': ( 'pandas.html#better_pdataframe_from_frame',
                                                                                 'nbstata/pandas.py')},
            'nbstata.stata': { 'nbstata.stata.drop_var': ('stata.html#drop_var', 'nbstata/stata.py'),
                               'nbstata.stata.get_global': ('stata.html#get_global', 'nbstata/stata.py'),
                               'nbstata.stata.get_local': ('stata.html#get_local', 'nbstata/stata.py'),
                               'nbstata.stata.get_scalar': ('stata.html#get_scalar', 'nbstata/stata.py'),
                               'nbstata.stata.macro_expand': ('stata.html#macro_expand', 'nbstata/stata.py'),
                               'nbstata.stata.obs_count': ('stata.html#obs_count', 'nbstata/stata.py'),
                               'nbstata.stata.pwd': ('stata.html#pwd', 'nbstata/stata.py'),
                               'nbstata.stata.run_direct': ('stata.html#run_direct', 'nbstata/stata.py'),
                               'nbstata.stata.run_single': ('stata.html#run_single', 'nbstata/stata.py'),
                               'nbstata.stata.set_local': ('stata.html#set_local', 'nbstata/stata.py'),
                               'nbstata.stata.stata_formatted': ('stata.html#stata_formatted', 'nbstata/stata.py'),
                               'nbstata.stata.variable_names': ('stata.html#variable_names', 'nbstata/stata.py')},
            'nbstata.stata_more': { 'nbstata.stata_more.IndexVar': ('stata_more.html#indexvar', 'nbstata/stata_more.py'),
                                    'nbstata.stata_more.IndexVar.__enter__': ( 'stata_more.html#indexvar.__enter__',
                                                                               'nbstata/stata_more.py'),
                                    'nbstata.stata_more.IndexVar.__exit__': ('stata_more.html#indexvar.__exit__', 'nbstata/stata_more.py'),
                                    'nbstata.stata_more.SelectVar': ('stata_more.html#selectvar', 'nbstata/stata_more.py'),
                                    'nbstata.stata_more.SelectVar.__enter__': ( 'stata_more.html#selectvar.__enter__',
                                                                                'nbstata/stata_more.py'),
                                    'nbstata.stata_more.SelectVar.__exit__': ( 'stata_more.html#selectvar.__exit__',
                                                                               'nbstata/stata_more.py'),
                                    'nbstata.stata_more.SelectVar.__init__': ( 'stata_more.html#selectvar.__init__',
                                                                               'nbstata/stata_more.py'),
                                    'nbstata.stata_more.SelectVar.clear': ('stata_more.html#selectvar.clear', 'nbstata/stata_more.py'),
                                    'nbstata.stata_more.diverted_stata_output': ( 'stata_more.html#diverted_stata_output',
                                                                                  'nbstata/stata_more.py'),
                                    'nbstata.stata_more.diverted_stata_output_quicker': ( 'stata_more.html#diverted_stata_output_quicker',
                                                                                          'nbstata/stata_more.py'),
                                    'nbstata.stata_more.get_local_dict': ('stata_more.html#get_local_dict', 'nbstata/stata_more.py'),
                                    'nbstata.stata_more.local_names': ('stata_more.html#local_names', 'nbstata/stata_more.py'),
                                    'nbstata.stata_more.locals_code_from_dict': ( 'stata_more.html#locals_code_from_dict',
                                                                                  'nbstata/stata_more.py'),
                                    'nbstata.stata_more.run_as_program': ('stata_more.html#run_as_program', 'nbstata/stata_more.py'),
                                    'nbstata.stata_more.run_direct_cleaned': ( 'stata_more.html#run_direct_cleaned',
                                                                               'nbstata/stata_more.py'),
                                    'nbstata.stata_more.run_sfi': ('stata_more.html#run_sfi', 'nbstata/stata_more.py'),
                                    'nbstata.stata_more.user_expression': ('stata_more.html#user_expression', 'nbstata/stata_more.py'),
                                    'nbstata.stata_more.var_from_varlist': ('stata_more.html#var_from_varlist', 'nbstata/stata_more.py')},
            'nbstata.stata_session': { 'nbstata.stata_session.StataSession': ( 'stata_session.html#statasession',
                                                                               'nbstata/stata_session.py'),
                                       'nbstata.stata_session.StataSession.__init__': ( 'stata_session.html#statasession.__init__',
                                                                                        'nbstata/stata_session.py'),
                                       'nbstata.stata_session.StataSession._compile_re': ( 'stata_session.html#statasession._compile_re',
                                                                                           'nbstata/stata_session.py'),
                                       'nbstata.stata_session.StataSession._completions': ( 'stata_session.html#statasession._completions',
                                                                                            'nbstata/stata_session.py'),
                                       'nbstata.stata_session.StataSession._get_locals': ( 'stata_session.html#statasession._get_locals',
                                                                                           'nbstata/stata_session.py'),
                                       'nbstata.stata_session.StataSession._post_run_hook': ( 'stata_session.html#statasession._post_run_hook',
                                                                                              'nbstata/stata_session.py'),
                                       'nbstata.stata_session.StataSession._run_as_program_w_locals': ( 'stata_session.html#statasession._run_as_program_w_locals',
                                                                                                        'nbstata/stata_session.py'),
                                       'nbstata.stata_session.StataSession._update_ending_delimiter': ( 'stata_session.html#statasession._update_ending_delimiter',
                                                                                                        'nbstata/stata_session.py'),
                                       'nbstata.stata_session.StataSession.clear_suggestions': ( 'stata_session.html#statasession.clear_suggestions',
                                                                                                 'nbstata/stata_session.py'),
                                       'nbstata.stata_session.StataSession.dispatch_run': ( 'stata_session.html#statasession.dispatch_run',
                                                                                            'nbstata/stata_session.py'),
                                       'nbstata.stata_session.StataSession.get_local_dict': ( 'stata_session.html#statasession.get_local_dict',
                                                                                              'nbstata/stata_session.py'),
                                       'nbstata.stata_session.StataSession.get_suggestions': ( 'stata_session.html#statasession.get_suggestions',
                                                                                               'nbstata/stata_session.py'),
                                       'nbstata.stata_session.StataSession.refresh_suggestions': ( 'stata_session.html#statasession.refresh_suggestions',
                                                                                                   'nbstata/stata_session.py'),
                                       'nbstata.stata_session._run_simple': ('stata_session.html#_run_simple', 'nbstata/stata_session.py'),
                                       'nbstata.stata_session.warn_re_unclosed_comment_block_if_needed': ( 'stata_session.html#warn_re_unclosed_comment_block_if_needed',
                                                                                                           'nbstata/stata_session.py')}}}
